@@ -751,6 +751,70 @@ topics: List[TopicDescriptor] = [
         name="Grid voltage {phase} to {next_phase}",
         metric_type=MetricType.VOLTAGE,
     ),
+    # Heatpump topics
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/Current",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_current",
+        name="heatpump current",
+        metric_type=MetricType.CURRENT,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/Energy/Forward",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_energy_forward",
+        name="heatpump consumption",
+        metric_type=MetricType.ENERGY,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/Frequency",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_frequency",
+        name="heatpump frequency",
+        metric_type=MetricType.FREQUENCY,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/Power",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_power",
+        name="heatpump power",
+        metric_type=MetricType.POWER,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/Voltage",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_voltage",
+        name="heatpump voltage",
+        metric_type=MetricType.VOLTAGE,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/{phase}/Current",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_current_{phase}",
+        name="heatpump current on {phase}",
+        metric_type=MetricType.CURRENT,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/{phase}/Energy/Forward",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_energy_forward_{phase}",
+        name="heatpump consumption on {phase}",
+        metric_type=MetricType.ENERGY,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/{phase}/Power",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_power_{phase}",
+        name="heatpump power on {phase}",
+        metric_type=MetricType.POWER,
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/heatpump/{device_id}/Ac/{phase}/Voltage",
+        message_type=MetricKind.SENSOR,
+        short_id="heatpump_voltage_{phase}",
+        name="heatpump voltage on {phase}",
+        metric_type=MetricType.VOLTAGE,
+    ),
     # Inverter topics (Like Phoenix)
     TopicDescriptor(
         topic="N/{installation_id}/inverter/{device_id}/Ac/Out/{phase}/I",
@@ -1706,6 +1770,26 @@ topics: List[TopicDescriptor] = [
         name="DC Battery Power",
         metric_type=MetricType.POWER,
     ),
+    TopicDescriptor(
+        topic="$$func/system/system_dc_battery_charge_power",
+        depends_on=["system_dc_battery_power"],
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_charge_energy",
+        name="DC Battery Charge Energy",
+        metric_type=MetricType.ENERGY,
+        precision=3,
+        experimental=True
+    ),
+    TopicDescriptor(
+        topic="$$func/system/system_dc_battery_discharge_power",
+        depends_on=["system_dc_battery_power"],
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_discharge_energy",
+        name="DC Battery Discharge Energy",
+        metric_type=MetricType.ENERGY,
+        precision=3,
+        experimental=True
+    ),
     TopicDescriptor(
         topic="N/{installation_id}/system/{device_id}/Dc/Battery/Soc",
         message_type=MetricKind.SENSOR,
@@ -1727,6 +1811,15 @@ topics: List[TopicDescriptor] = [
         name="PV Power",
         metric_type=MetricType.POWER,
     ),
+    TopicDescriptor(
+        topic="$$func/system/left_riemann_sum",
+        depends_on=["system_dc_pv_power"],
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_pv_energy",
+        name="PV Energy",
+        metric_type=MetricType.ENERGY,
+        precision=3,
+    ),
     TopicDescriptor(
         topic="N/{installation_id}/system/{device_id}/Dc/System/Power",
         message_type=MetricKind.SENSOR,
