@@ -8,7 +8,7 @@ import random
 import ssl
 import re
 import string
-from typing import Any, Callable, Optional
+from typing import Any, Callable, Optional, Tuple
 
 import paho.mqtt.client as mqtt
 from paho.mqtt.client import Client as MQTTClient, PayloadType
@@ -77,8 +77,63 @@ class Hub:
         topic_log_info: str | None = None,
         operation_mode: OperationMode = OperationMode.FULL,
         device_type_exclude_filter: list[DeviceType] | None = None,
+        update_frequency_seconds: int | None = None
     ) -> None:
-        """Initialize."""
+        """
+        Initialize a Hub instance for communicating with a Venus OS MQTT broker.
+
+        Parameters
+        ----------
+        host: str
+            MQTT broker hostname or IP address.
+        port: int
+            MQTT broker port (1-65535).
+        username: str | None
+            Username for MQTT authentication, or None for anonymous access.
+        password: str | None
+            Password for MQTT authentication, or None.
+        use_ssl: bool
+            If True, an SSL/TLS context will be configured when connecting.
+        installation_id: str | None
+            If provided, used to replace `{installation_id}` placeholders in topics.
+            If None, the installation id will be discovered from the broker when
+            `connect()` is called.
+        model_name: str | None
+            Optional device model name used for informational purposes.
+        serial: str | None
+            Optional device serial identifier (defaults to "noserial").
+        topic_prefix: str | None
+            Optional prefix that is prepended to every subscribe/publish topic.
+        topic_log_info: str | None
+            Optional substring used to elevate logging level for matching topics.
+        operation_mode: OperationMode
+            Controls which TopicDescriptor entries are active (e.g. FULL, READ_ONLY,
+            EXPERIMENTAL).
+        device_type_exclude_filter: list[DeviceType] | None
+            Optional list of device types to exclude from subscriptions.
+        update_frequency_seconds: int | None
+            Optional update frequency used by metrics.
+            if None = Update only when source data change
+            if 0 = Update as new mqtt data received
+            if > 0 = Update at the specified interval (in seconds) even if no new data is received
+
+        Behavior
+        --------
+        - Performs strict parameter validation and raises `ValueError` or `TypeError`
+          for invalid values or types.
+        - Builds internal topic maps and subscription lists from the package-level
+          `topics` list, applying operation mode and device type filters.
+        - Generates a unique MQTT client id and creates (but does not connect)
+          a paho-mqtt Client instance. No network operations occur during
+          initialization; call `connect()` to establish the MQTT connection.
+
+        Raises
+        ------
+        ValueError
+            If `host` is empty or `port` is out of the valid range.
+        TypeError
+            If an argument has an incorrect type.
+        """
         global running_client_id
         self._instance_id = running_client_id
         running_client_id += 1
@@ -118,9 +173,11 @@ class Hub:
             for device_type in device_type_exclude_filter:
                 if not isinstance(device_type, DeviceType):
                     raise TypeError(f"device_type_filter must contain only DeviceType instances, got type={type(device_type).__name__}, value={device_type!r}")
+        if update_frequency_seconds is not None and not isinstance(update_frequency_seconds, int):
+            raise TypeError(f"update_frequency_seconds must be an integer or None, got type={type(update_frequency_seconds).__name__}, value={update_frequency_seconds!r}")
         _LOGGER.info(
-            "Initializing Hub[ID: %d](host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s, operation_mode=%s, device_type_exclude_filter=%s)",
-            self._instance_id, host, port, username, use_ssl, installation_id, model_name, topic_prefix, operation_mode, device_type_exclude_filter
+            "Initializing Hub[ID: %d](host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s, operation_mode=%s, device_type_exclude_filter=%s, update_frequency_seconds=%s)",
+            self._instance_id, host, port, username, use_ssl, installation_id, model_name, topic_prefix, operation_mode, device_type_exclude_filter, update_frequency_seconds
         )
         self._model_name = model_name
         self.host = host
@@ -152,6 +209,7 @@ class Hub:
         self._all_metrics: dict[str, Metric] = {}
         self._first_connect = True
         self._first_full_publish = True
+        self._update_frequency_seconds = update_frequency_seconds
 
         # Filter the active topics
         metrics_active_topics: list[TopicDescriptor] = []
@@ -210,9 +268,12 @@ class Hub:
             [desc for desc in expanded_topics if desc.is_adjustable_suffix],
             lambda desc: Hub._remove_placeholders_map(merge_is_adjustable_suffix(desc))
         )
-        subscription_list1 = [Hub._remove_placeholders(topic.topic) for topic in expanded_topics]
-        subscription_list2 = [Hub._remove_placeholders(merge_is_adjustable_suffix(desc)) for desc in expanded_topics if desc.is_adjustable_suffix]
+        subscription_list1 = [Hub._remove_placeholders(topic.topic) for topic in expanded_topics if not topic.is_formula]
+        subscription_list2 = [Hub._remove_placeholders(merge_is_adjustable_suffix(topic)) for topic in expanded_topics if topic.is_adjustable_suffix and not topic.is_formula]
         self._subscription_list = subscription_list1 + subscription_list2
+        self._pending_formula_topics: list[TopicDescriptor] = [topic for topic in expanded_topics if topic.is_formula]
+        for topic in self._pending_formula_topics:
+            _LOGGER.info("Formula topic detected: %s", topic.topic)
         self._client = MQTTClient(callback_api_version=CallbackAPIVersion.VERSION2, client_id=self._client_id)
         _LOGGER.info("Hub initialized. Client ID: %s", self._client_id)
 
@@ -341,6 +402,19 @@ class Hub:
 
         self._handle_normal_message(topic, payload, log_debug)
 
+    def is_dependency_met(self, topic: TopicDescriptor, relevant_device: Device) -> Tuple[bool, list[Metric]]:
+        if not topic.depends_on:
+            return True, []
+        dependencies: list[Metric] = []
+        for dependency in topic.depends_on:
+            metric_name = f"{relevant_device.short_unique_id}_{dependency}"
+            dependency_metric = self._all_metrics.get(metric_name)
+            if dependency_metric is None:
+                _LOGGER.debug("Formula topic %s is missing dependency metric: %s", topic.topic, metric_name)
+                return False, []
+            dependencies.append(dependency_metric)
+        return True, dependencies
+
     def _handle_full_publish_message(self, payload: str) -> None:
         """Handle full publish message."""
         echo = self.get_keepalive_echo(payload)
@@ -358,12 +432,34 @@ class Hub:
         _LOGGER.debug("Full publish completed: %s", echo)
         new_metrics: list[tuple[Device, Metric]] = []
         for metric_placeholder in self._metrics_placeholders:
-            metric = metric_placeholder.device.add_placeholder(metric_placeholder, self._all_metrics, self._fallback_placeholders, self)
-            self._all_metrics[metric.unique_id] = metric
+            metric = metric_placeholder.device.add_placeholder(metric_placeholder, self._fallback_placeholders, self)
+            self._all_metrics[metric._hub_unique_id] = metric
             new_metrics.append((metric_placeholder.device, metric))
+        self._metrics_placeholders.clear()
+        self._fallback_placeholders.clear()
+        # Activate formula entities
+        if len(new_metrics) > 0:
+            new_formula_metrics: list[tuple[Device, Metric]] = []
+            for topic in self._pending_formula_topics:
+                _LOGGER.debug("Trying to resolve formula topic: %s", topic)
+                relevant_devices: list[Device] = [device for device in self._devices.values() if device.device_type.code == topic.topic.split("/")[1]]
+                for device in relevant_devices:
+                    is_met, dependencies = self.is_dependency_met(topic, device)
+                    if is_met:
+                        _LOGGER.info("Formula topic resolved: %s", topic)
+                        metric = device.add_formula_metric(topic, self)
+                        depends_on: dict[str, Metric] = {}
+                        for dependency_metric in dependencies:
+                            dependency_metric.add_dependency(metric)
+                            depends_on[dependency_metric._hub_unique_id] = dependency_metric
+                        metric.init(depends_on, self._loop, _LOGGER.debug)
+                        _LOGGER.info("Formula metric created: %s", metric)
+                        new_formula_metrics.append((device, metric))
+            # Send all new metrics to clients
+            new_metrics.extend(new_formula_metrics)
         # We are sending the new metrics now as we can be sure that the metric handled all the attribute topics and now ready.
         for device, metric in new_metrics:
-            metric.phase2_init(device.unique_id, self._all_metrics)
+            metric.phase2_init(device.short_unique_id, self._all_metrics)
             try:
                 if callable(self._on_new_metric):
                     if self._loop.is_running():
@@ -371,29 +467,28 @@ class Hub:
                         self._loop.call_soon_threadsafe(self._on_new_metric, self, device, metric)
             except Exception as exc:
                 _LOGGER.error("Error calling _on_new_metric callback %s", exc, exc_info=True)
-        self._metrics_placeholders.clear()
-        self._fallback_placeholders.clear()
         # Trace the version once
         if self._first_full_publish:
-            platform_device = self._devices.get(f"{self.installation_id}_system_0")
-            if platform_device:
-                version_metric = platform_device.get_metric_from_unique_id(f"{self.installation_id}_system_0_platform_firmware_installed_version")
-                if version_metric and version_metric.value:
-                    if version_metric.value[0] == "v":
-                        try:
-                            firmware_version = float(version_metric.value[1:])
-                            if firmware_version < 3.5:
-                                _LOGGER.warning("Firmware version is below v3.5: %s", version_metric.value)
-                            else:
-                                _LOGGER.info("Firmware version is good enough: %s", version_metric.value)
-                        except (ValueError, TypeError):
-                            _LOGGER.error("Firmware version format not float: %s", version_metric.value)
-                    else:
-                        _LOGGER.error("Firmware version format not supported: %s", version_metric.value)
+            version_metric_name = "system_0_platform_venus_firmware_installed_version"
+            version_metric = self._all_metrics.get(version_metric_name)
+            if version_metric and version_metric.value:
+                if version_metric.value[0] == "v":
+                    try:
+                        # Accept versions like 'v3.70' and 'v3.70~15' by stripping any '~' suffix
+                        ver_str = version_metric.value[1:]
+                        if "~" in ver_str:
+                            ver_str = ver_str.split("~", 1)[0]
+                        firmware_version = float(ver_str)
+                        if firmware_version < 3.5:
+                            _LOGGER.warning("Firmware version is below v3.5: %s", version_metric.value)
+                        else:
+                            _LOGGER.info("Firmware version is good enough: %s", version_metric.value)
+                    except (ValueError, TypeError):
+                        _LOGGER.error("Firmware version format not float: %s", version_metric.value)
                 else:
-                    _LOGGER.warning("Version metric not found: %s", self._installation_id)
+                    _LOGGER.error("Firmware version format not supported: %s", version_metric.value)
             else:
-                _LOGGER.warning("System device not found: %s", self._installation_id)
+                _LOGGER.warning("Version metric not found: %s", version_metric_name)
         if self._loop.is_running():
             self._loop.call_soon_threadsafe(self._first_refresh_event.set)
         self._first_full_publish = False
@@ -439,7 +534,7 @@ class Hub:
                 return
 
         device = self._get_or_create_device(parsed_topic, desc)
-        metric_place_holder = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, log_debug)
+        metric_place_holder = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, log_debug, self)
         if isinstance(metric_place_holder, MetricPlaceholder):
             self._metrics_placeholders.append(metric_place_holder)
         elif isinstance(metric_place_holder, FallbackPlaceholder):
@@ -486,10 +581,15 @@ class Hub:
     async def _keepalive_loop(self) -> None:
         """Run keepalive every 30 seconds."""
         _LOGGER.info("Starting keepalive loop")
+        count = 0
         while True:
             try:
                 await self._keepalive()
                 await asyncio.sleep(30)
+                # We should keep alive all metrics every 60 seconds
+                count += 1
+                if count % 2 == 0:
+                    self._keepalive_metrics()
             except asyncio.CancelledError:
                 _LOGGER.info("Keepalive loop canceled")
                 raise
@@ -497,6 +597,12 @@ class Hub:
                 _LOGGER.error("Error in keepalive loop: %s", exc, exc_info=True)
                 await asyncio.sleep(5)  # Short delay before retrying
 
+    def _keepalive_metrics(self) -> None:
+        """Keep alive all metrics."""
+        _LOGGER.debug("Keeping alive all metrics")
+        for metric in self._all_metrics.values():
+            metric._keepalive(self._loop, _LOGGER.debug)
+
     def _start_keep_alive_loop(self) -> None:
         """Start the keep_alive loop."""
         _LOGGER.info("Creating keepalive task")
@@ -689,41 +795,37 @@ class Hub:
 
     def _get_or_create_device(self, parsed_topic: ParsedTopic, desc: TopicDescriptor) -> Device:
         """Get or create a device based on topic."""
-        unique_id = self._create_device_unique_id(
+        full_unique_id, short_unique_id = self._create_device_unique_id(
             parsed_topic.installation_id,
             parsed_topic.device_type.code,
             parsed_topic.device_id,
         )
-        device = self._devices.get(unique_id)
+        device = self._devices.get(short_unique_id)
         if device is None:
-            _LOGGER.info("Creating new device: unique_id=%s, parsed_topic=%s", unique_id, parsed_topic)
+            _LOGGER.info("Creating new device: unique_id=%s, parsed_topic=%s", full_unique_id, parsed_topic)
             device = Device(
-                unique_id,
+                full_unique_id,
+                short_unique_id,
                 parsed_topic,
                 desc,
             )
-            self._devices[unique_id] = device
+            self._devices[short_unique_id] = device
         return device
 
-    def _create_device_unique_id(self, installation_id: str, device_type: str, device_id: str) -> str:
+    def _create_device_unique_id(self, installation_id: str, device_type: str, device_id: str) -> tuple[str, str]:
         """Create a unique ID for a device."""
-        unique_id = f"{installation_id}_{device_type}_{device_id}"
-        return unique_id
-
-    def get_device_from_unique_id(self, unique_id: str) -> Device:
-        """Get a device from a unique id."""
-        device = self._devices.get(unique_id)
-        if device is None:
-            raise KeyError(f"Device with unique id {unique_id} not found.")
-        return device
+        short_unique_id = f"{device_type}_{device_id}"
+        full_unique_id = f"{installation_id}_{short_unique_id}"
+        return full_unique_id, short_unique_id
 
     def _get_device_unique_id_from_metric_unique_id(self, unique_id: str) -> str:
         return "_".join(unique_id.split("_")[:3])
 
     def get_metric_from_unique_id(self, unique_id: str) -> Metric | None:
         """Get a metric from a unique id."""
-        device = self.get_device_from_unique_id(self._get_device_unique_id_from_metric_unique_id(unique_id))
-        return device.get_metric_from_unique_id(unique_id)
+        id_without_installation = unique_id.split("_", 1)[-1]
+        metric = self._all_metrics.get(id_without_installation)
+        return metric
 
     @property
     def devices(self) -> dict[str, Device]:
@@ -790,7 +892,7 @@ class Hub:
         return self._on_new_metric
 
     @on_new_metric.setter
-    def on_new_metric(self, value: CallbackOnNewMetric):
+    def on_new_metric(self, value: CallbackOnNewMetric | None):
         """Sets the on_new_metric callback."""
         self._on_new_metric = value
 
