@@ -7,36 +7,49 @@ from __future__ import annotations
 
 import asyncio
 from collections.abc import Callable
+import time
 import logging
+from typing import TYPE_CHECKING, Any
 
 from .id_utils import replace_complex_id_to_simple, replace_complex_ids
 from .constants import MetricKind, MetricNature, MetricType
 from .data_classes import ParsedTopic, TopicDescriptor
 
+if TYPE_CHECKING:
+    from .hub import Hub
+    from .device import Device
+    from .formula_metric import FormulaMetric
+
 _LOGGER = logging.getLogger(__name__)
 
+CallbackOnUpdate = Callable[["Metric", Any], None]
 
 class Metric:
     """Representation of a Victron Venus sensor."""
 
-    def __init__(self, unique_id: str, name: str, descriptor: TopicDescriptor, parsed_topic: ParsedTopic) -> None:
+    def __init__(self, device: Device, unique_id: str, name: str, descriptor: TopicDescriptor, short_id: str, key_values: dict[str, str], hub: Hub) -> None:
         """Initialize the sensor."""
         _LOGGER.debug(
             "Creating new metric: unique_id=%s, type=%s, nature=%s",
             unique_id, descriptor.metric_type, descriptor.metric_nature
         )
         assert descriptor.name is not None, "name must be set for metric"
-        assert parsed_topic.device_type is not None, "device_type must be set for metric"
-        
+        self._device = device
         self._descriptor = descriptor
         self._unique_id = unique_id
         self._value = None
-        self._short_id = parsed_topic.short_id
+        self._short_id = short_id
         self._name = name
-        self._key_values: dict[str, str] = parsed_topic.key_values
-        self._on_update: Callable | None = None
+        self._key_values: dict[str, str] = key_values
+        self._on_update: CallbackOnUpdate | None = None
         self._generic_name: str | None = None
         self._generic_short_id: str | None = None
+        self._depend_on_me: list[FormulaMetric] = []
+        self._hub_unique_id: str = f"{device.short_unique_id}_{self._short_id}"
+        self._hub = hub
+        self._last_update: float = 0
+        self._last_seen: float = 0
+
         _LOGGER.debug("Metric %s initialized", repr(self))
 
     def __str__(self) -> str:
@@ -54,6 +67,9 @@ class Metric:
             f"{key_values_part})"
             )
 
+    def __repr__(self) -> str:
+        return self.__str__()
+
     def phase2_init(self, device_id: str, all_metrics: dict[str, Metric]) -> None:
         """Second phase of initializing the metric."""
         assert self._descriptor.name is not None, f"name must be set for topic: {self._descriptor.topic}"
@@ -81,16 +97,24 @@ class Metric:
 
         return ParsedTopic.replace_ids(orig_str, self.key_values)
 
-    @property
-    def formatted_value(self) -> str:
+    def add_dependency(self, formula_metric: FormulaMetric) -> None:
+        """Add a dependency to the metric."""
+        self._depend_on_me.append(formula_metric)
+
+    def format_value(self, value) -> str:
         """Returns the formatted value of the metric."""
-        if self._value is None:
+        if value is None:
             return ""
 
         if self._descriptor.unit_of_measurement is None:
-            return str(self._value)
+            return str(value)
         else:
-            return f"{self._value} {self._descriptor.unit_of_measurement}"
+            return f"{value} {self._descriptor.unit_of_measurement}"
+
+    @property
+    def formatted_value(self):
+        """Returns the value of the metric."""
+        return self.format_value(self._value)
 
     @property
     def value(self):
@@ -156,40 +180,72 @@ class Metric:
         return self._key_values
 
     @property
-    def on_update(self) -> Callable | None:
+    def on_update(self) -> CallbackOnUpdate | None:
         """Returns the on_update callback."""
         return self._on_update
 
     @on_update.setter
-    def on_update(self, value: Callable):
+    def on_update(self, value: CallbackOnUpdate | None) -> None:
         """Sets the on_update callback."""
         self._on_update = value
 
+    def _keepalive(self, event_loop: asyncio.AbstractEventLoop | None, log_debug: Callable[..., None]):
+        """Reset metrics value if no updates or send last values if they got skipped"""
+        silence_timeout = min(60, self._hub._update_frequency_seconds * 3) if self._hub._update_frequency_seconds is not None else 60
+        now = time.time()
+        elapsed = now - self._last_seen
+        if elapsed > silence_timeout:
+            log_debug("Metric %s has been silent for %.2fs, resetting", self.unique_id, elapsed)
+            self._handle_message(None, event_loop, log_debug)
+        elif self._last_seen > self._last_update:
+            log_debug("Metric %s has been updated at %.2f but not published since %.2fs, re-publishing", self.unique_id, self._last_update, self._last_seen)
+            self._handle_message(self._value, event_loop, log_debug)
+
+
     def _handle_message(self, value, event_loop: asyncio.AbstractEventLoop | None, log_debug: Callable[..., None]):
         """Handle a message."""
+        now = time.time()
+        self._last_seen = now
+
+        changed = False
         if value != self._value:
             log_debug(
                 "Metric %s value changed: %s -> %s %s",
                 self.unique_id, self._value, value,
                 self._descriptor.unit_of_measurement or ''
             )
+            changed = True
+            self._value = value
         else:
             log_debug(
                 "Metric %s value unchanged: %s %s",
                 self.unique_id, value,
                 self._descriptor.unit_of_measurement or ''
             )
-            return
 
-        self._value = value
+        # In case of zero update frequency, always consider changed when MQTT message is received
+        if self._hub._update_frequency_seconds == 0:
+            changed = True
+        # In case of non-zero update frequency, respect the update frequency limit only for numerical values
+        elif self._hub._update_frequency_seconds is not None and isinstance(value, (float, int)):
+            elapsed = now - self._last_update
+            if elapsed < self._hub._update_frequency_seconds:
+                _LOGGER.debug(
+                    "Update for %s skipped due to frequency limit (%.2fs < %ds)",
+                    self.unique_id, elapsed, self._hub._update_frequency_seconds
+                )
+                changed = False
+            else:
+                changed = True
 
-        if not event_loop:
-            return
+        if changed and event_loop and callable(self._on_update) and event_loop.is_running():
+            self._last_update = now
+            try:
+                # If the event loop is running, schedule the callback
+                event_loop.call_soon_threadsafe(self._on_update, self, self.value)
+            except Exception as exc:
+                log_debug("Error calling callback %s", exc, exc_info=True)
 
-        try:
-            if callable(self._on_update):
-                if event_loop.is_running():
-                    # If the event loop is running, schedule the callback
-                    event_loop.call_soon_threadsafe(self._on_update, self)
-        except Exception as exc:
-            log_debug("Error calling callback %s", exc, exc_info=True)
+        for dependency in self._depend_on_me:
+            assert self != dependency, f"Circular dependency detected: {self}"
+            dependency._handle_formula(event_loop, log_debug)
